/**
    Componente tare-conexe
    Algoritmul Kosaraju
    Complexitate:
       2 * m => O(m)  - timp de executare
       2 * m => O(m)  - spatiu de memorie ocupat
*/
#include <fstream>
#include <vector>
#include <stack>
#include <set>
using namespace std;
using SI  = set<int>;
using SSI  = set<SI>;
using VB  = vector<bool>;
using VI  = vector<int>;
using VVI = vector<VI>;

ifstream fin("ctc.in");
ofstream fout("ctc.out");

VVI G, Gt;  /// graful / graful transpus
VB v;       /// marccam nodurile vizitate
SSI ctc;    /// retine componentele tare conexe
SI c;       /// comp tare conexa curenta
stack<int> stk;  /// retine nodurile in ordine totpologica
int n, m;

void CitesteGraf();
void Kosaraju();
void Dfs(int x);
void DfsT(int x);
void ScrieCTC();

int main()
{
    CitesteGraf();
    Kosaraju();
    ScrieCTC();
}

void Kosaraju()
{
    v = VB(n + 1);
    for (int x = 1; x <= n; ++x) /// O(m)
    {
        if (v[x]) continue;
        Dfs(x);
    }

    v = VB(n + 1);

    while (!stk.empty())   /// O(m)
    {
        int x = stk.top();
        stk.pop();
        if (v[x]) continue;
        c.clear();
        DfsT(x);
        ctc.insert(c);
    }
}

void Dfs(int x) // O(n*n)
{
    v[x] = true;
    for (const int& y : G[x])
    {
        if (v[y]) continue;
        Dfs(y);
    }
    stk.push(x);
}

void DfsT(int x)
{
    v[x] = true;
    c.insert(x);
    for (int const& y : Gt[x])
    {
        if (v[y]) continue;
        DfsT(y);
    }
}

void ScrieCTC()
{
    fout << ctc.size() << '\n';
    for (auto comp : ctc)
    {
        for (int x : comp)
            fout << x << ' ';
        fout << '\n';
    }
}

void CitesteGraf()
{
    fin >> n >> m;
    G = Gt = VVI(n + 1);
    int x, y;
    while (m--)
    {
        fin >> x >> y;
        G[x].push_back(y);
        Gt[y].push_back(x);
    }
}
