/**
    Componente tare-conexe
    Algoritmul lui Tarjan
    Complexitate:
       m => O(m)  - timp de executare
       m => O(m)  - spatiu de memorie ocupat
*/
#include <fstream>
#include <vector>
#include <stack>
#include <set>
using namespace std;
using SI  = set<int>;
using SSI  = set<SI>;
using VB  = vector<bool>;
using VI  = vector<int>;
using VVI = vector<VI>;

ifstream fin("ctc.in");
ofstream fout("ctc.out");

VVI G;  /// graful / graful transpus
SSI ctc;    /// retine componentele tare conexe
SI c;       /// comp tare conexa curenta
VI index;   /// index[x] = indexul (timpul de descoperire a lui x)
VI low;     /// low[x] = indexul minim a care se poate
            /// pornind din nodul x si folosind o singura muchie de intoarcere
int idx;    /// indexul curent
stack<int> stk;  /// retine nodurile in ordinea descoperirii acestora
VB inStack; /// inStack[x] = true, daca x este in stiva
int n, m;

void CitesteGraf();
void Tarjan();
void Dfs(int x);
void ScrieCTC();
void ExtractCTC(int x);

int main()
{
    CitesteGraf();
    Tarjan();
    ScrieCTC();
}

void Tarjan()
{
    for (int x = 1; x <= n; ++x)
        if (!index[x])
            Dfs(x);
}

void Dfs(int x) // O(m)
{
    index[x] = low[x] = ++idx;
    stk.push(x);
    inStack[x] = true;

    for (int y : G[x])
        if (!index[y])  // nod nevizitat, deci y este fiul lui x
        {
            Dfs(y);     // x-y tree edge
            low[x] = min(low[x], low[y]);
        }
        else  // nod vizitat
            if (inStack[y])  // back-edge
                low[x] = min(low[y], index[y]);

    if (low[x] == index[x])
        ExtractCTC(x);
}

void ExtractCTC(int x)
{
    c.clear();
    while (!stk.empty())
    {
        int y = stk.top();
        stk.pop();
        inStack[y] = false;
        c.insert(y);
        if (y == x)
            break;
    }

    ctc.insert(c);
}

void ScrieCTC()
{
    fout << ctc.size() << '\n';
    for (auto comp : ctc)
    {
        for (int x : comp)
            fout << x << ' ';
        fout << '\n';
    }
}

void CitesteGraf()
{
    fin >> n >> m;
    index = low = VI(n + 1);
    inStack = VB(n + 1);
    G = VVI(n + 1);
    int x, y;
    while (m--)
    {
        fin >> x >> y;
        G[x].push_back(y);
    }
}
